      subroutine DYNdgz_mp(norder)
 
C +------------------------------------------------------------------------+
C | MAR DYNAMICS   FAST                                    08-09-2017  MAR |
C |   SubRoutine DYNdgz includes in the Horizontal Momentum Equations      |
C |              the terms representing the Pressure Gradient Force (PGF)  |
C |                                                                        |
C +------------------------------------------------------------------------+
C |                                                                        |
C |   INPUT   (via common block)                                           |
C |   ^^^^^    brocam: Brown and Campana Time Scheme Switch                |
C |            itFast: Short             Time Step  Counter                |
C |                                                                        |
C |            uairDY, vairDY, pktaDY   : u, v, and P / Time Step n        |
C |            ubefDY, vbefDY, ddux,ddvx: u, v        / Time Step n-1, n-2 |
C |            uairDY: x-wind speed component                        (m/s) |
C |            vairDY: y-wind speed component                        (m/s) |
C |            pktaDY: potential temperature divided by 100.[kPa]**(R/Cp)  |
C |             virDY: Contribution from Air Loading               (kg/kg) |
C |                                                                        |
C |   OUTPUT  (via common block)                                           |
C |   ^^^^^^   uairDY, vairDY, pktaDY: u, v, and P Values / Time Step n+1  |
C |            gpmiDY(i,j,k) = g * z (i,j,k-1/2),  (Geopotential)  (m2/s2) |
C |                                                                        |
C |   METHOD:  1) Solves the Hydrostatic Relation:                         |
C |   ^^^^^^       0     =- 1 /rho -dPHI/dp                                |
C |                      => gives the Geopotential PHI between Sigma Levels|
C |            2) Solves the Contributions       :                         |
C |                du/dt:=         -dPHI/dx                                |
C |                dv/dt:=         -dPHI/dy                                |
C |            3) Spatial  Numerical Scheme      :                         |
C |                Spatial Discretisation on Arakawa A Grid                |
C |   norder.EQ.2  2th Order Accurate Horizontal Spatial Differencing .OR. |
C |   norder.NE.2  4th Order Accurate Horizontal Spatial Differencing      |
C |                dPHI/dx and dPHI/dy are computed on p**(R/Cp) Surfaces  |
C |            4) Temporal Numerical Scheme      :                         |
C |                Time Split (i.e. each contribution computed separately) |
C |                Split Time Differencing, i.e. pressure Evolution and    |
C |                PGF are computed on Short Time Step dtfast=dt/(ntFast+1)|
C |                Advection and Diffusion are Computed on a Longer One)   |
C |                Brown and Campana Time Scheme used over Short Time Step |
C |                                                                        |
C |   REFER.:  1) Purser and Leslie, MWR 116, 2069--2080, 1988     (A Grid)|
C |   ^^^^^^   2) Marchuk, Numer.Meth.in Weath.Predict.,  1974 (Time Split)|
C |            3) Gadd, QJRMS 104, 569--582, 1978 (Split Time Differencing)|
C |            4) Brown and Campana, MWR 106, 1125--1136, 1978             |
C |                                                                        |
C +------------------------------------------------------------------------+
 
 
      IMPLICIT NONE
 
 
C +--Global Variables
C +  ================
 
      include 'MARCTR.inc'
      include 'MARphy.inc'
 
      include 'MARdim.inc'
      include 'MARgrd.inc'
 
      include 'MAR_DY.inc'
      include 'MAR_UB.inc'
 
      include 'MAR_WK.inc'
 
      integer  norder
 
 
C +--Local  Variables
C +  ================
 
      include 'MARvec.inc'
 
      real     ff_dgz(mx,my,mz)              ! Momentum Dummy Variable
      real     adv_uu(mx,my,mz)              ! Advected     u-Momentum
      real     adv_vv(mx,my,mz)              ! Advected     v-Momentum
 
      real     bca   ,bcb   ,c1a   ,c1b   ,ddux  ,ddvx  ,fraCLS,sigCLS
      real     Raylei
 
 
C +--DATA
C +  ====
 
      data bca/0.245e0/,bcb/0.510e0/
C +...Parameters of the Brown-Campana (1978, MWR, p.1125) scheme
C +   WARNING : scheme is unstable for bca maximum value (0.25)
 
 
C +--Contributions   from Momentum Advection
C +  =======================================
 
C +          **********
        call DYNadv_dLF_mp(norder,uairDY,vairDY,adv_uu,adv_vv)
C +          **********
 
C +--Integration of the Hydrostatic Equation
C +  =======================================
 
C +--EXNER Potential
C +  ---------------
 
      IF (brocam)                                                   THEN
 
        DO j=1,my
        DO i=1,mx
          WKxy4(i,j)=exp(cap*log(pstDYn(i,j)              +ptopDY))
          WKxy1(i,j)=     cp*     WKxy4(i,j)
        END DO
        END DO
 
      ELSE
 
        DO j=1,my
        DO i=1,mx
          WKxy4(i,j)=exp(cap*log(pstDY (i,j)              +ptopDY))
          WKxy1(i,j)=     cp*     WKxy4(i,j)
        END DO
        END DO
 
      END IF
 
C +--Surface     Contribution to Exner Function
C +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        fraCLS    = 0.0d+0
c #IL   fraCLS    = 0.5d+0
C +...  fraCLS    : CLS Fraction where Temperature~Surface Temperature
C +   0<fraCLS<1    and generally close to zero
C +     fraCLS    = 0.5  ==> linear variation of potential Temperature
C +                          is assumed beween levels k=mz and k=mzz
C +
        sigCLS    =(1.0d+0 - fraCLS) + fraCLS  *sigma(mz)
 
      IF (brocam)                                                   THEN
        DO j=1,my
        DO i=1,mx
          WKxy5(i,j)=cp * exp(cap*log(pstDYn(i,j)*sigCLS       +ptopDY))
        END DO
        END DO
      ELSE
        DO j=1,my
        DO i=1,mx
          WKxy5(i,j)=cp * exp(cap*log(pstDY (i,j)*sigCLS       +ptopDY))
        END DO
        END DO
      END IF
 
C +--Atmospheric Contribution to Exner Function
C +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
!$OMP PARALLEL DO private (i,j,k)
      DO j=1,my
      DO k=1,mz
 
        IF (k.ge.mz.and.ptopDY.le.0.0)                              THEN
c            DO j=1,my
            DO i=1,mx
              WKxy3(i,j)=0.0
            END DO
c            END DO
        ELSE
          IF (brocam)                                               THEN
c            DO j=1,my
            DO i=1,mx
              WKxy3(i,j)=exp(cap*log(pstDYn(i,j)*sigmid(mzz-k)+ptopDY))
            END DO
c            END DO
          ELSE
c            DO j=1,my
            DO i=1,mx
              WKxy3(i,j)=exp(cap*log(pstDY (i,j)*sigmid(mzz-k)+ptopDY))
            END DO
c            END DO
          END IF
        END IF
 
c            DO j=1,my
            DO i=1,mx
              WKxy2(i,j)       = cp*WKxy3(i,j)
              WKxyz4(i,j,mzz-k)=    WKxy3(i,j)
C +...        WKxyz4           :    p ** (R/Cp)
 
            END DO
c            END DO
 
 
C +--GEO---Potential (Mid Layer k-1/2)
C +  ---------------
 
C +--Of    the Surface Layer
C +  ~~~~~~~~~~~~~~~~~~~~~~~
        IF (k.eq.1)                                                 THEN
 
c          DO j=1,my
          DO i=1,mx
            gpmiDY(i,j,mz)=gplvDY(i,j,mzz)
     .                 +((WKxy5(i,j)-WKxy2(i,j))*pktaDY(i,j,mz)
     .                  +(WKxy1(i,j)-WKxy5(i,j))*pktaDY(i,j,mzz))
     .                                      *(1.0+virDY(i,j,mz))
C +...      REMARK : It is assumed that the Geopotential Difference
C +                  in Lower Layer depends only on pktaDY at 1st Sigma Lev.
 
          END DO
c          END DO
 
        ELSE
 
C +--Above the Surface Layer
C +  ~~~~~~~~~~~~~~~~~~~~~~~
c          DO j=1,my
          DO i=1,mx
            gpmiDY(i,j,mzz-k)=gpmiDY(i,j,mzz+1-k)
     .                     +(WKxy1(i,j)-WKxy2(i,j))*pktaDY(i,j,mzz-k)
     .                                         *(1.0+virDY(i,j,mzz-k))
          END DO
c          END DO
 
        END IF
 
c          DO j=1,my
          DO i=1,mx
            WKxy1(i,j)=WKxy2(i,j)
          END DO
c          END DO
 
      END DO
      END DO
!$OMP END PARALLEL DO
 
 
C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
C +--Update of u and v at the Lateral Boundaries
C +  ===========================================
 
!$OMP PARALLEL DO
!$OMP. private (i,j,k,WKxy1,WKxy3,c1a,c1b,ddux,ddvx)
        DO   k=1,mz
          DO j=1,my
            ubefDY( 1,j,k) = uairDY( 1,j,k)
            ubefDY(mx,j,k) = uairDY(mx,j,k)
            vbefDY( 1,j,k) = vairDY( 1,j,k)
            vbefDY(mx,j,k) = vairDY(mx,j,k)
          END DO
c       END DO
      IF (mmy.gt.1)                                                 THEN
c       DO   k=1,mz
          DO i=1,mx
            ubefDY(i, 1,k) = uairDY(i, 1,k)
            ubefDY(i,my,k) = uairDY(i,my,k)
            vbefDY(i, 1,k) = vairDY(i, 1,k)
            vbefDY(i,my,k) = vairDY(i,my,k)
          END DO
c       END DO
      END IF
 
C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
C +--Mesoscale Geopotential Gradient
C +  ===============================
 
c     DO k=1,mz
 
C +--For Hydrostatic Contribution
C +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        IF (k.eq.mz)                                          GO TO 30
          DO j=1,my
          DO i=1,mx
            WKxy1( i,j)  = WKxyz4(i,j,k+1)
            WKxyz1(i,j,k)= gpmiDY(i,j,k+1)
          END DO
          END DO
                                                              GO TO 31
 30     CONTINUE
          DO j=1,my
          DO i=1,mx
            WKxy1( i,j)  = WKxy4(i,j)
            WKxyz1(i,j,k)= gplvDY(i,j,mzz)
          END DO
          END DO
 31     CONTINUE
 
        DO j=1,my
        DO i=1,mx
          WKxy3( i,j)  = WKxyz4(i,j,k)-WKxy1(i,j)
        END DO
        END DO
 
 
C +--Gradient following x
C +  --------------------
 
C +--For Hydrostatic Contribution
C +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        DO j=jp11,my1
        DO i=   1,mx
            WKxyz2(i,j,k)=(WKxyz4(i,j,k)*WKxyz1(i,j,k)
     .                     -WKxy1(i,j)  *gpmiDY(i,j,k))/WKxy3(i,j)
            WKxyz3(i,j,k)=(gpmiDY(i,j,k)-WKxyz1(i,j,k))/WKxy3(i,j)
        END DO
        END DO
 
C +--For Hydrostatic Contribution
C +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        IF (norder.EQ.2)                                            THEN
          DO i=   1,mx
          DO j=jp11,my1
              WKxyz5(i,j,k)=
     .                 (WKxyz2(ip1(i),j,k)-WKxyz2(im1(i),j,k)) *dxinv2
              WKxyz6(i,j,k)=
     .                 (WKxyz3(ip1(i),j,k)-WKxyz3(im1(i),j,k)) *dxinv2
          END DO
          END DO
        ELSE
          DO i=   1,mx
          DO j=jp11,my1
              WKxyz5(i,j,k)=
     .          fac43 *(WKxyz2(ip1(i),j,k)-WKxyz2(im1(i),j,k)
     .         -0.125 *(WKxyz2(ip2(i),j,k)-WKxyz2(im2(i),j,k)))*dxinv2
              WKxyz6(i,j,k)=
     .          fac43 *(WKxyz3(ip1(i),j,k)-WKxyz3(im1(i),j,k)
     .         -0.125 *(WKxyz3(ip2(i),j,k)-WKxyz3(im2(i),j,k)))*dxinv2
          END DO
          END DO
        ENDIF
 
 
C +--Contribution to u Wind Speed Component
C +  --------------------------------------
 
C +- First Step
C +  ~~~~~~~~~~
          IF (itFast.eq.1)                                          THEN
            DO j=jp11,my1
            DO i=ip11,mx1
              c1a        =-(WKxyz5(i,j,k)+pkDY(i,j,k)*WKxyz6(i,j,k))
     .                    + adv_uu(i,j,k)
 
              c1b           =  dtfast *             c1a
              ubefDY(i,j,k) =  uairDY(i,j,k) +      c1b
              uairDY(i,j,k) =  uairDY(i,j,k) + c1b +c1b
 
              dg1xDY(i,j,k) =  c1a
              dgzxDY(i,j,k) =  c1a
            END DO
            END DO
 
 
C +- Next  Step
C +  ~~~~~~~~~~
          ELSE
            IF (itFast.le.ntFast)                                   THEN
              IF (brocam)                                           THEN
                DO j=jp11,my1
                DO i=ip11,mx1
                  c1a        =-(WKxyz5(i,j,k)+pkDY(i,j,k)*WKxyz6(i,j,k))
     .                       +  adv_uu(i,j,k)
 
                  ddux          =  ubefDY(i,j,k)
                  ubefDY(i,j,k) =  uairDY(i,j,k)
                  uairDY(i,j,k) =  ddux   + 2.0*     dtfast
     .                                    *(bcb*     dgzxDY(i,j,k)
     .                                     +bca*(c1a+dg1xDY(i,j,k)))
C +...            U (n+1)       =  U(n-1) + 2     Dt  (Du/Dt)
 
C +- Robert Time Filter
C +  ~~~~~~~~~~~~~~~~~~
c #rt             ubefDY(i,j,k) =  ubefDY(i,j,k)
c #rt.              + Robert*(0.5*(uairDY(i,j,k)+ddux) -  ubefDY(i,j,k))
 
                  dg1xDY(i,j,k) =  dgzxDY(i,j,k)
                  dgzxDY(i,j,k) =  c1a
                END DO
                END DO
              ELSE
                DO j=jp11,my1
                DO i=ip11,mx1
                  c1a        =-(WKxyz5(i,j,k)+pkDY(i,j,k)*WKxyz6(i,j,k))
     .                        + adv_uu(i,j,k)
 
                  ddux          =  ubefDY(i,j,k)
                  ubefDY(i,j,k) =  uairDY(i,j,k)
                  uairDY(i,j,k) =  ddux   + 2.0  *dtfast *c1a
C +...            U (n+1)       =  U(n-1) + 2     Dt     (Du/Dt)
 
                END DO
                END DO
              END IF
 
C +- Last  Step
C +  ~~~~~~~~~~
            ELSE
              IF (brocam) then
                DO j=jp11,my1
                DO i=ip11,mx1
                  c1a        =-(WKxyz5(i,j,k)+pkDY(i,j,k)*WKxyz6(i,j,k))
     .                       +  adv_uu(i,j,k)
 
                  ddux          =  ubefDY(i,j,k)
                  ubefDY(i,j,k) =  uairDY(i,j,k)
                  uairDY(i,j,k) =  ddux   +    dtfast
     .                                      *(bcb*     dgzxDY(i,j,k)
     .                                       +bca*(c1a+dg1xDY(i,j,k)))
C +...            U (n+1)       =  U(n)   +    Dt     (Du/Dt)'
C +               Leapfrog-Backward (e.g. Haltiner and Williams, p.152)
 
                  dg1xDY(i,j,k) =  dgzxDY(i,j,k)
                  dgzxDY(i,j,k) =  c1a
                END DO
                END DO
              ELSE
                DO j=jp11,my1
                DO i=ip11,mx1
                  c1a        =-(WKxyz5(i,j,k)+pkDY(i,j,k)*WKxyz6(i,j,k))
     .                       +  adv_uu(i,j,k)
 
                  ddux          =  ubefDY(i,j,k)
                  ubefDY(i,j,k) =  uairDY(i,j,k)
                  uairDY(i,j,k) =  ddux   +    dtfast *c1a
C +...            U (n+1)       =  U(n)   +    Dt     (Du/Dt)'
C +               Leapfrog-Backward (e.g. Haltiner and Williams, p.152)
 
                END DO
                END DO
              END IF
            END IF
          END IF
 
 
C +- Gradient following y
C +  --------------------
 
        IF (mmy.gt.1)                                               THEN
 
          DO j=   1,my
            WKxyz2( 1,j,k) = 0.
            WKxyz2(mx,j,k) = 0.
            WKxyz3( 1,j,k) = 0.
            WKxyz3(mx,j,k) = 0.
          ENDDO
 
C +--For Hydrostatic Contribution
C +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
          DO i=ip11,mx1
          DO j=   1,my
            WKxyz2(i,j,k)=(WKxyz4(i,j,k)*WKxyz1(i,j,k)
     .                     -WKxy1(i,j)  *gpmiDY(i,j,k))/WKxy3(i,j)
            WKxyz3(i,j,k)=(gpmiDY(i,j,k)-WKxyz1(i,j,k))/WKxy3(i,j)
 
          END DO
          END DO
 
C +--For Hydrostatic Contribution
C +  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
         IF (norder.EQ.2)                                           THEN
          DO j=   1,my
          DO i=ip11,mx1
            WKxyz5(i,j,k)=
     .                  (WKxyz2(i,jp1(j),k)-WKxyz2(i,jm1(j),k) )*dyinv2
            WKxyz6(i,j,k)=
     .                  (WKxyz3(i,jp1(j),k)-WKxyz3(i,jm1(j),k) )*dyinv2
          END DO
          END DO
         ELSE
          DO j=   1,my
          DO i=ip11,mx1
            WKxyz5(i,j,k)=
     .           fac43 *(WKxyz2(i,jp1(j),k)-WKxyz2(i,jm1(j),k)
     .          -0.125 *(WKxyz2(i,jp2(j),k)-WKxyz2(i,jm2(j),k)))*dyinv2
            WKxyz6(i,j,k)=
     .           fac43 *(WKxyz3(i,jp1(j),k)-WKxyz3(i,jm1(j),k)
     .          -0.125 *(WKxyz3(i,jp2(j),k)-WKxyz3(i,jm2(j),k)))*dyinv2
          END DO
          END DO
         ENDIF
 
 
C +--Contribution to v Wind Speed Component
C +  --------------------------------------
 
C +- First Step
C +  ~~~~~~~~~~
          IF (itFast.eq.1)                                          THEN
            DO j=jp11,my1
            DO i=ip11,mx1
              c1a        =-(WKxyz5(i,j,k)+pkDY(i,j,k)*WKxyz6(i,j,k))
     .                   +  adv_vv(i,j,k)
 
              c1b        = dtfast *                 c1a
              vbefDY(i,j,k) =  vairDY(i,j,k) +      c1b
              vairDY(i,j,k) =  vairDY(i,j,k) + c1b +c1b
 
              dg1yDY(i,j,k) = c1a
              dgzyDY(i,j,k) = c1a
            END DO
            END DO
 
C +- Next  Step
C +  ~~~~~~~~~~
          ELSE
            IF   (itFast.le.ntFast)                                 THEN
              IF (brocam)                                           THEN
                DO j=jp11,my1
                DO i=ip11,mx1
                  c1a     = -(WKxyz5(i,j,k)+pkDY(i,j,k)*WKxyz6(i,j,k))
     .                    +   adv_vv(i,j,k)
 
                  ddvx          = vbefDY(i,j,k)
                  vbefDY(i,j,k) = vairDY(i,j,k)
                  vairDY(i,j,k) = ddvx    +   2.0*     dtfast
     .                                      *(bcb*     dgzyDY(i,j,k)
     .                                       +bca*(c1a+dg1yDY(i,j,k)))
C +               V (n+1)       = V(n-1)  +   2    Dt  (Dv/Dt)
 
C +- Robert Time Filter
C +  ~~~~~~~~~~~~~~~~~~
c #rt             vbefDY(i,j,k) =  vbefDY(i,j,k)
c #rt.              + Robert*(0.5*(vairDY(i,j,k)+ddvx) -vbefDY(i,j,k))
 
                  dg1yDY(i,j,k) =  dgzyDY(i,j,k)
                  dgzyDY(i,j,k) =  c1a
                END DO
                END DO
              ELSE
                DO j=jp11,my1
                DO i=ip11,mx1
                  c1a     = -(WKxyz5(i,j,k)+pkDY(i,j,k)*WKxyz6(i,j,k))
     .                    +   adv_vv(i,j,k)
 
                  ddvx          = vbefDY(i,j,k)
                  vbefDY(i,j,k) = vairDY(i,j,k)
                  vairDY(i,j,k) = ddvx    + 2.0*dtfast *c1a
C +               V (n+1)       = V(n-1)  + 2   Dt   (Dv/Dt)
 
                END DO
                END DO
              END IF
 
C +- Last  Step
C +  ~~~~~~~~~~
            ELSE
              IF (brocam)                                           THEN
                DO j=jp11,my1
                DO i=ip11,mx1
                  c1a     = -(WKxyz5(i,j,k)+pkDY(i,j,k)*WKxyz6(i,j,k))
     .                    +   adv_vv(i,j,k)
 
                  ddvx          = vbefDY(i,j,k)
                  vbefDY(i,j,k) = vairDY(i,j,k)
                  vairDY(i,j,k) = ddvx    +   dtfast
     .                                      *(bcb*     dgzyDY(i,j,k)
     .                                       +bca*(c1a+dg1yDY(i,j,k)))
C +...            V (n+1)       =  V(n)   +   Dt (Dv/Dt)'
C +               Leapfrog-Backward (e.g. Haltiner and Williams, p.152)
 
                  dg1yDY(i,j,k) = dgzyDY(i,j,k)
                  dgzyDY(i,j,k) = c1a
                END DO
                END DO
              ELSE
                DO j=jp11,my1
                DO i=ip11,mx1
                  c1a     = -(WKxyz5(i,j,k)+pkDY(i,j,k)*WKxyz6(i,j,k))
     .                    +   adv_vv(i,j,k)
 
                  ddvx          = vbefDY(i,j,k)
                  vbefDY(i,j,k) = vairDY(i,j,k)
                  vairDY(i,j,k) = ddvx    +    dtfast *c1a
C +...            V (n+1)       =  V(n)   +    Dt (Dv/Dt)'
C +               Leapfrog-Backward (e.g. Haltiner and Williams, p.152)
 
                END DO
                END DO
              END IF
            END IF
          END IF
        END IF
 
C +--Rayleigh Friction (Ref. ARPS 4.0 User's Guide, para 6.4.3 p.152)
C +  =================
 
      if(k<= mzabso) then
        DO j=1,my
        DO i=1,mx
          uairDY(i,j,k)=(uairDY(i,j,k) + Ray_UB(k)*dtFast*uairUB(i,j,k))
     .                 /(1.0           + Ray_UB(k)*dtFast              )
          vairDY(i,j,k)=(vairDY(i,j,k) + Ray_UB(k)*dtFast*vairUB(i,j,k))
     .                 /(1.0           + Ray_UB(k)*dtFast              )
        END DO
        END DO
      ENDIF
      ENDDO
!$OMP END PARALLEL DO
      return
      end
